@@ -17,8 +17,8 @@ use Scalar::Util qw'blessed weaken';
 use Encode qw(encode decode);
 use Carp qw(carp croak );
 
-use vars qw'$VERSION %link_spec';
-$VERSION = '0.58';
+use vars qw'$VERSION %link_spec @CARP_NOT';
+$VERSION = '0.59';
 
 =head1 NAME
 
@@ -411,7 +411,7 @@ JS
     };
     
     my $eval_in_sandbox = $self->repl->declare(<<'JS', 'list');
-    function (w,d,str,env) {
+    function (w,d,str,env,caller,line) {
         var unsafeWin = w.wrappedJSObject;
         var safeWin = XPCNativeWrapper(unsafeWin);
         var sandbox = Components.utils.Sandbox(safeWin);
@@ -423,15 +423,21 @@ JS
             sandbox.window[e] = env[e]
         }
         sandbox.__proto__ = unsafeWin;
-        var res = Components.utils.evalInSandbox(str, sandbox);
+
+        var res = Components.utils.evalInSandbox(str, sandbox, "1.8",caller,line);
         return [res,typeof(res)];
     };
 JS
     $window ||= $self->tab->{linkedBrowser}->{contentWindow};
     # Report errors from scope of caller
-    local @MozRepl::RemoteObject::Instance::CARP_NOT = (@MozRepl::RemoteObject::Instance::CARP_NOT,__PACKAGE__);
-    #warn Dumper @MozRepl::RemoteObject::CARP_NOT;
-    return $eval_in_sandbox->($window,$doc,$str,$js_env);
+    # This feels weirdly backwards here, but oh well:
+    @CARP_NOT = (ref $self->repl); # we trust this
+    #local @MozRepl::RemoteObject::Instance::CARP_NOT = (@MozRepl::RemoteObject::Instance::CARP_NOT,__PACKAGE__);
+    #local @MozRepl::RemoteObject::CARP_NOT = (@MozRepl::RemoteObject::CARP_NOT,__PACKAGE__);
+    
+    my ($caller,$line) = (caller)[1,2];
+    
+    $eval_in_sandbox->($window,$doc,$str,$js_env,$caller,$line);
 };
 *eval = \&eval_in_page;
 
@@ -3048,8 +3054,7 @@ sub is_visible {
     return
         unless $options{ dom };
     $options{ window } ||= $self->tab->{linkedBrowser}->{contentWindow};
-    
-    
+
     my $_is_visible = $self->repl->declare(<<'JS');
     function (obj,window)
     {
@@ -3077,7 +3082,7 @@ sub is_visible {
                 if (style.visibility == 'hidden') {
                     return false;
                 };
-            }
+            };
             obj = obj.parentNode;
         };
         // The object does not live in the DOM at all
@@ -3114,6 +3119,15 @@ intervals are possible.
 
 =back
 
+Note that when passing in a selector, that selector is requeried
+on every poll instance. So the following query will work as expected:
+
+  xpath => '//*[contains(text(),"stand by")]'
+
+This also means that if your selector query relies on finding
+a changing text, you need to pass the node explicitly instead of
+passing the selector.
+
 =cut
 
 sub wait_until_invisible {
@@ -3128,23 +3142,25 @@ sub wait_until_invisible {
     
     _default_limiter( 'maybe', \%options );
 
-    if (! $options{dom}) {
-        $options{dom} = $self->_option_query(%options);
-    };
-    return
-        unless $options{dom};
 
     my $timeout_after;
     if ($timeout) {
         $timeout_after = time + $timeout;
     };
     my $v;
-    while (     $v = $self->is_visible($options{dom})
-           and (!$timeout_after or time < $timeout_after )) {
+    my $node;
+    do {
+        $node = $options{ dom };
+        if (! $node) {
+            $node = $self->_option_query(%options);
+        };
+        return
+            unless $node;
         sleep $sleep;
-    };
-    if (! $v and time > $timeout_after) {
-        croak "Timeout of $timeout seconds reached while waiting for element";
+    } while (     $v = $self->is_visible($node)
+           and (!$timeout_after or time < $timeout_after ));
+    if ($node and time >= $timeout_after) {
+        croak "Timeout of $timeout seconds reached while waiting for element to become invisible";
     };    
 };
 
@@ -3234,7 +3250,7 @@ sub expand_frames {
 
 =head1 IMAGE METHODS
 
-=head2 C<< $mech->content_as_png( [$tab, \%coordinates ] ) >>
+=head2 C<< $mech->content_as_png( [$tab, \%coordinates, \%target_size ] ) >>
 
     my $png_data = $mech->content_as_png();
 
@@ -3254,9 +3270,25 @@ If the coordinates are given, that rectangle will be cut out.
 The coordinates should be a hash with the four usual entries,
 C<left>,C<top>,C<width>,C<height>.
 
+=item *
+
+The target size of the image can also be given. It defaults to
+the size of the image. The allowed parameters in the hash are
+
+C<scalex>, C<scaley> - for specifying the scale, default is 1.0 in each direction.
+
+C<width>, C<height> - for specifying the target size
+
+If you want the resulting image to be 480 pixels wide, specify
+
+    { width: 480 }
+
+The height will then be calculated from the ratio of original width to
+original height.
+
 =back
 
-This is specific to WWW::Mechanize::Firefox.
+This method is specific to WWW::Mechanize::Firefox.
 
 Currently, the data transfer between Firefox and Perl
 is done Base64-encoded. It would be beneficial to find what's
@@ -3265,14 +3297,16 @@ necessary to make JSON handle binary data more gracefully.
 =cut
 
 sub content_as_png {
-    my ($self, $tab, $rect) = @_;
+    my ($self, $tab, $rect, $target_rect) = @_;
     $tab ||= $self->tab;
     $rect ||= {};
+    $target_rect ||= {};
     
     # Mostly taken from
     # http://wiki.github.com/bard/mozrepl/interactor-screenshot-server
+    # Except for the addition of a target image size
     my $screenshot = $self->repl->declare(<<'JS');
-    function (tab,rect) {
+    function (tab,rect,target_rect) {
         var browser = tab.linkedBrowser;
         var browserWindow = Components.classes['@mozilla.org/appshell/window-mediator;1']
             .getService(Components.interfaces.nsIWindowMediator)
@@ -3289,12 +3323,34 @@ sub content_as_png {
         var top = rect.top || 0;
         var width = rect.width || body.clientWidth;
         var height = rect.height || body.clientHeight;
-        canvas.width = width;
-        canvas.height = height;
+        
+        if( isNaN( target_rect.scalex * target_rect.scaley )) {
+            // No scale was given
+            // Was a fixed target width / height given?
+            if( target_rect.width ) {
+                target_rect.scalex = target_rect.width / width;
+            };
+            if( target_rect.height ) {
+                target_rect.scaley = target_rect.height / height
+            };
+
+            // If only one of scalex / scaley is given, force the other
+            // to be the same, default to 1.0
+            target_rect.scalex = target_rect.scalex || target_rect.scaley || (target_rect.width / width) || 1.0;
+            target_rect.scaley = target_rect.scaley || target_rect.scalex || (target_rect.height / height) || 1.0;
+        } else {
+            // alert("scales fixed");
+        };
+        // Calculate the target width/height if missing:
+        target_rect.height = target_rect.height || height * target_rect.scaley;
+        target_rect.width  = target_rect.width  || width * target_rect.scalex;
+        
+        canvas.width = target_rect.width;
+        canvas.height = target_rect.height;
         var ctx = canvas.getContext('2d');
-        ctx.clearRect(0, 0, width, height);
+        ctx.clearRect(0, 0, target_rect.width, target_rect.height);
         ctx.save();
-        ctx.scale(1.0, 1.0);
+        ctx.scale(target_rect.scalex, target_rect.scaley);
         ctx.drawWindow(win, left, top, width, height, 'rgb(255,255,255)');
         ctx.restore();
 
@@ -3305,7 +3361,7 @@ sub content_as_png {
         // );
     }
 JS
-    my $scr = $screenshot->($tab, $rect);
+    my $scr = $screenshot->($tab, $rect, $target_rect);
     return $scr ? decode_base64($scr) : undef
 };
 
